"""Trip Query."""
# pylint: disable=no-member, unused-argument, too-many-locals, duplicate-code, wildcard-import, unused-wildcard-import, cyclic-import


# Autogenerated
# DO NOT MODIFY

from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple, Union

from taegis_sdk_python import GraphQLNoRowsInResultSetError
from taegis_sdk_python._consts import TaegisEnum
from taegis_sdk_python.services.trip.types import *
from taegis_sdk_python.utils import (
    build_output_string,
    parse_union_result,
    prepare_input,
)

if TYPE_CHECKING:  # pragma: no cover
    from taegis_sdk_python.services.trip import TripService


log = logging.getLogger(__name__)


class TaegisSDKTripQuery:
    """Taegis Trip Query operations."""

    def __init__(self, service: TripService):
        self.service = service

    def list_active_api_products(
        self, application: Union[ApiProductGroup, TaegisEnum]
    ) -> List[ApiProduct]:
        """None."""
        endpoint = "listActiveApiProducts"

        result = self.service.execute_query(
            endpoint=endpoint,
            variables={
                "application": prepare_input(application),
            },
            output=build_output_string(ApiProduct),
        )
        if result.get(endpoint) is not None:
            return ApiProduct.schema().load(
                [r or {} for r in result.get(endpoint)], many=True
            )
        raise GraphQLNoRowsInResultSetError("for query listActiveApiProducts")

    def list_api_integrations(self) -> List[ApiIntegrationSummary]:
        """None."""
        endpoint = "listApiIntegrations"

        result = self.service.execute_query(
            endpoint=endpoint,
            variables={},
            output=build_output_string(ApiIntegrationSummary),
        )
        if result.get(endpoint) is not None:
            return ApiIntegrationSummary.schema().load(
                [r or {} for r in result.get(endpoint)], many=True
            )
        raise GraphQLNoRowsInResultSetError("for query listApiIntegrations")

    def api_integration(self, id_: int) -> ApiIntegration:
        """None."""
        endpoint = "apiIntegration"

        result = self.service.execute_query(
            endpoint=endpoint,
            variables={
                "id": prepare_input(id_),
            },
            output=build_output_string(ApiIntegration),
        )
        if result.get(endpoint) is not None:
            return ApiIntegration.from_dict(result.get(endpoint))
        raise GraphQLNoRowsInResultSetError("for query apiIntegration")

    def list_api_integration_history(
        self, id_: int, start_time: str, end_time: str, page_number: int, page_size: int
    ) -> ApiIntegrationHistoryPage:
        """None."""
        endpoint = "listApiIntegrationHistory"

        result = self.service.execute_query(
            endpoint=endpoint,
            variables={
                "id": prepare_input(id_),
                "startTime": prepare_input(start_time),
                "endTime": prepare_input(end_time),
                "pageNumber": prepare_input(page_number),
                "pageSize": prepare_input(page_size),
            },
            output=build_output_string(ApiIntegrationHistoryPage),
        )
        if result.get(endpoint) is not None:
            return ApiIntegrationHistoryPage.from_dict(result.get(endpoint))
        raise GraphQLNoRowsInResultSetError("for query listApiIntegrationHistory")

    def get_form_options(self, product_id: int, form_item_name: str) -> List[str]:
        """None."""
        endpoint = "getFormOptions"

        result = self.service.execute_query(
            endpoint=endpoint,
            variables={
                "productId": prepare_input(product_id),
                "formItemName": prepare_input(form_item_name),
            },
            output="",
        )
        if result.get(endpoint) is not None:
            return result.get(endpoint)
        raise GraphQLNoRowsInResultSetError("for query getFormOptions")

    def get_env_config(self) -> List[TripEnvConfig]:
        """None."""
        endpoint = "getEnvConfig"

        result = self.service.execute_query(
            endpoint=endpoint, variables={}, output=build_output_string(TripEnvConfig)
        )
        if result.get(endpoint) is not None:
            return TripEnvConfig.schema().load(
                [r or {} for r in result.get(endpoint)], many=True
            )
        raise GraphQLNoRowsInResultSetError("for query getEnvConfig")
